@@ -5,8 +5,9 @@ import os
 import tornado.template
 import tornado.web
 
+from dorothy.settings import settings
 from dorothy.lib import xjson
-from dorothy.lib.exceptions import ApiError
+from dorothy.lib.exceptions import ApiError, SettingsError
 
 
 class MainBaseHandler(tornado.web.RequestHandler):
@@ -15,6 +16,8 @@ class MainBaseHandler(tornado.web.RequestHandler):
 
         self.query = {}
         self.body = {}
+        self._page_data = dict()
+        self._redirect = None
 
     ##################################################
     # The follow method are what should be implemented
@@ -58,25 +61,28 @@ class MainBaseHandler(tornado.web.RequestHandler):
         '''
         pass
 
-    def _handle_request_exception(self, e):
-        if isinstance(e, tornado.web.HTTPError):
-            response = e.log_message
-            code = e.status_code
-        elif isinstance(e, ApiError):
-            response = e.message
-            code = e.code
-        else:
-            code = 500
-            response = str(e)
-    
-        # TODO: add some logging here
-        # TODO: if in debug mode, raise all exceptions
-    
-        return self._respond((code, response))
+    # def _handle_request_exception(self, e):
+    #     if isinstance(e, tornado.web.HTTPError):
+    #         response = e.log_message
+    #         code = e.status_code
+    #     elif isinstance(e, ApiError):
+    #         response = e.message
+    #         code = e.code
+    #     else:
+    #         code = 500
+    #         response = str(e)
+    # 
+    #     # TODO: add some logging here
+    #     # TODO: if in debug mode, raise all exceptions
+    # 
+    #     return self._respond((code, response))
 
 
     ##################################################
 
+    def redirect(self, url, permanent=False, status=None):
+        self._redirect = (url, permanent, status)
+
     def get(self, *args, **kwargs):
         return self._process(*args, **kwargs)
 
@@ -121,10 +127,28 @@ class MainBaseHandler(tornado.web.RequestHandler):
 
         self.on_finish_all()
 
+    def template(self, tmpl, data=None):
+        data = data or {}
+        pth = tmpl.strip('/')
+        if len(pth.split('/')) == 1:
+            pth = ''
+        else:
+            pth = '/'.join(pth.split('/')[:-1] + [''])
+        tmpl = tmpl.replace('/', '.')
+
+        if not settings.get('template_path'):
+            raise SettingsError('Can not load template files. No template_path given.')
+
+        self._page_data = data
+        loader = tornado.template.Loader(settings.template_path)
+        data.update(self.get_template_namespace())
+        return loader.load('{0}{1}.html'.format(pth, tmpl)).generate(**data)
+
     def _process(self, *args, **kwargs):
         self._parse_query()
         self._parse_body()
 
+        args = tuple([a for a in args if a])
         if self.method == 'get':
             response = self.get_(*args, **kwargs)
         elif self.method == 'post':
@@ -165,21 +189,43 @@ class MainBaseHandler(tornado.web.RequestHandler):
 
 
 class WebBaseHandler(MainBaseHandler):
-    def template(self, tmpl, data=None):
-        data = data or {}
+    def __init__(self, application, request, **kwargs):
+        super(WebBaseHandler, self).__init__(application, request, **kwargs)
 
-        loader = tornado.template.Loader('{0}/../../views'.format(os.path.dirname(os.path.abspath(__file__))))
-        return loader.load('{0}.html'.format(tmpl)).generate(**data)
+        self._js = []
+        self._css = []
+
+    def js(self, *args):
+        for js in args:
+            self._js.append(js)
+
+    def css(self, *args):
+        for css in args:
+            self._css.append(css)
+
+    @property
+    def js(self):
+        return self._js
+
+    @property
+    def css(self):
+        return self._css
 
     def _respond(self, response):
+        if self._redirect:
+            return tornado.web.RequestHandler.redirect(self, *self._redirect)
+
         if not isinstance(response, tuple):
             response = (200, response)
         status_code, content = response
 
         self.set_status(status_code)
-        self.write(content)
+        self.write(self.prepare_page(content, self._page_data))
         self.finish()
 
+    def prepare_page(self, content, page_data):
+        return content
+
 
 class ApiBaseHandler(MainBaseHandler):
     def initialize(self):
@@ -221,6 +267,9 @@ class ApiBaseHandler(MainBaseHandler):
         raise ApiError(code, message)
 
     def _respond(self, response):
+        if self._redirect:
+            response = dict(redirect=self._redirect[0])
+
         if not isinstance(response, tuple):
             response = (200, response)
         status_code, content = response
